@@ -126,6 +126,10 @@ def get_simulator(model, envcond=None, constraints=None, reference=None, reset_s
         module_name, class_name = map_model_simulator[name]
         module = __import__(module_name, fromlist=[None])
         class_ = getattr(module, class_name)
+        try:
+            model.solver.configuration.timeout = ModelConstants.SOLVER_TIMEOUT
+        except:
+            pass
         instance = class_(model, envcond=envcond,
                           constraints=constraints, reference=reference, reset_solver=reset_solver)
     elif "etfl" in name:
@@ -133,6 +137,7 @@ def get_simulator(model, envcond=None, constraints=None, reference=None, reset_s
             from .cobra import Simulation
             from etfl.optim.config import standard_solver_config
             standard_solver_config(model, verbose=False)
+            model.solver.configuration.timeout = max(7200, ModelConstants.SOLVER_TIMEOUT)
             instance = Simulation(
                 model, envcond=envcond, constraints=constraints, reference=reference, reset_solver=reset_solver)
             instance._MAX_STR = 'max'
@@ -144,6 +149,7 @@ def get_simulator(model, envcond=None, constraints=None, reference=None, reset_s
             from cobra.core.model import Model
             if isinstance(model, Model):
                 from .cobra import Simulation
+                model.solver.configuration.timeout = ModelConstants.SOLVER_TIMEOUT
                 instance = Simulation(
                     model, envcond=envcond, constraints=constraints, reference=reference, reset_solver=reset_solver)
         except ImportError:
