@@ -61,7 +61,6 @@ def copy_func(f):
 class Dispatcher:
 
     def __init__(self):
-
         """
         Dispatcher for the simulate method of the Simulation interface
         It allows a simplification of the if else chain of methods provided as input to the simulate method
@@ -122,7 +121,7 @@ chemical_formula_re = re.compile('([A-Z][a-z]?)([0-9.]+[0-9.]?|(?=[A-Z])?)')
 
 
 def elements(formula):
-    all_elements = findall(chemical_formula_re, formula)
+    all_elements = re.findall(chemical_formula_re, formula)
     atoms = {}
     for atom, count in all_elements:
         if not count:
